/*
 * User: Tom
 * Date: 1-aug-2004
 * Time: 23:46:07
 */
package com.tombrus.imageConverter.jobs;

import com.tombrus.imageConverter.JobStep;
import com.tombrus.imageConverter.Job;

import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Rotate extends JobStep {
    private float angle;

    public int initFrom(Job job,String[] args,int i) {
        i = super.initFrom(job,args,i);
        i++;
        if (args.length <=i) {
            throw new Error("usage: -Rotate a");
        }
        angle = (float)Math.PI*Float.parseFloat(args[i])/180f;
        return i;
    }

    public void run() {
        final BufferedImage image = getJob().getImage();
        if (image !=null) {
            final int             w               = image.getWidth();
            final int             h               = image.getHeight();
            final AffineTransform rotateTransform = AffineTransform.getRotateInstance(angle,(double) w /2,(double) h /2);
            final Point2D         p1              = rotateTransform.transform(new Point2D.Float((float) 0,(float) 0),null);
            final Point2D         p2              = rotateTransform.transform(new Point2D.Float((float) 0,(float) h),null);
            final Point2D         p3              = rotateTransform.transform(new Point2D.Float((float) w,(float) 0),null);
            final Point2D         p4              = rotateTransform.transform(new Point2D.Float((float) w,(float) h),null);
            final double          minx            = Math.min(p1.getX(),Math.min(p2.getX(),Math.min(p3.getX(),p4.getX())));
            final double          maxx            = Math.max(p1.getX(),Math.max(p2.getX(),Math.max(p3.getX(),p4.getX())));
            final double          miny            = Math.min(p1.getY(),Math.min(p2.getY(),Math.min(p3.getY(),p4.getY())));
            final double          maxy            = Math.max(p1.getY(),Math.max(p2.getY(),Math.max(p3.getY(),p4.getY())));
            final int             neww            = (int) (maxx -minx);
            final int             newh            = (int) (maxy -miny);
            final AffineTransform moveAndRotate   = AffineTransform.getTranslateInstance(-minx,-miny);
            moveAndRotate.concatenate(rotateTransform);
            final BufferedImage newImage = new BufferedImage(neww,newh,image.getType());
            final Graphics2D    g         =   newImage.createGraphics();
            g.setColor(new Color(0,0,0,0));
            g.setTransform(moveAndRotate);
            g.drawImage(image,0,0,w,h,null);
            g.dispose();
            getJob().setImage(newImage);
        }
    }
}